/**
 * @author Mike Tung <miketung2013@gmail.com>
 * @license
 * Copyright Mike Tung All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file
 */
import { Component, Input, ViewChild } from '@angular/core';
import * as d3 from 'd3';
import { ChartService } from '../shared/chart.service';
import { DataService } from '../shared/data.service';
import { Tooltip } from '../shared/models/tooltip';
import { ToolTipService } from '../shared/tooltip.service';
export class BarChartComponent {
    /**
     * @param {?} chartService
     * @param {?} dataService
     * @param {?} tooltipService
     */
    constructor(chartService, dataService, tooltipService) {
        this.chartService = chartService;
        this.dataService = dataService;
        this.tooltipService = tooltipService;
        /**
         * Obtain the settings for making the chart here and define defaults if none are specified.
         */
        this.width = 400;
        this.height = 400;
        this.margins = { 'top': 60, 'right': 60, 'bottom': 60, 'left': 60 };
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        /*Check if data is given if not die*/
        if (this.data == null) {
            throw new Error('Missing Data!');
        }
        this.chartWidth = this.width - this.margins.left - this.margins.right;
        this.chartHeight = this.height - this.margins.top - this.margins.bottom;
        this.tooltipLabels = new Tooltip(this.x, this.y);
        this.chart = this.chartService.makeChartCanvas(this.chartContainer, this.width, this.height, this.margins);
        if (this.data) {
            this.draw(this.data);
        }
    }
    /**
     * This life cycle hook tells angular to detect any changes and to adjust
     * the chart accordingly and also serves to separate out different
     * instances of the same chart
     * @return {?}
     */
    ngOnChanges() {
        this.chartWidth = this.width - this.margins.left - this.margins.right;
        this.chartHeight = this.height - this.margins.top - this.margins.bottom;
        this.tooltipLabels = new Tooltip(this.x, this.y);
        if (this.chart && this.data) {
            this.draw(this.data);
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.width = null;
        this.height = null;
        this.margins = null;
        this.data = null;
        this.x = null;
        this.y = null;
        this.chartHeight = null;
        this.chartWidth = null;
        this.chart = null;
        this.xScale = null;
        this.yScale = null;
        this.xData = null;
        this.yData = null;
    }
    /**
     * This makes the axes for the chart.
     *
     *
     * @param {?} axis
     * @param {?} scale
     * @return {?}
     */
    makeAxis(axis, scale) {
        if (axis === 'x') {
            /* This is drawing the x-axis and adding a label*/
            this.chart.append('g')
                .attr('class', 'x-axis')
                .attr('transform', `translate(0, ${this.chartHeight})`)
                .call(d3.axisBottom(scale));
            const /** @type {?} */ movement = `${this.chartWidth / 2}, ${this.chartHeight + this.margins.bottom - 5}`;
            this.chart
                .append('text')
                .attr('transform', `translate(${movement})`)
                .text(`${this.x}`);
        }
        else if (axis === 'y') {
            /* This is drawing the y-axis and adding a label*/
            this.chart.append('g')
                .attr('class', 'y-axis')
                .call(d3.axisLeft(scale));
            this.chart.append('text')
                .attr('transform', 'rotate(-90)')
                .attr('y', `-${this.margins.left}`)
                .attr('x', `-${this.chartHeight / 2}`)
                .attr('dy', '1em')
                .style('text-anchor', 'middle')
                .text(`${this.y}`);
        }
    }
    /**
     * This is the main method to render the bar chart it operates on the d3
     * selection supplied and appends the scales, bars to the selection.
     *
     * @param {?} dataSet
     * @return {?}
     */
    draw(dataSet) {
        this.xData = this.dataService.getData(dataSet, this.x);
        this.yData = this.dataService.getData(dataSet, this.y);
        this.xScale = this.dataService.makeScale('categorical', this.xData, this.chartWidth);
        this.yScale = this.dataService.makeScale('linear', this.yData, this.chartHeight, false);
        this.makeAxis('x', this.xScale);
        this.makeAxis('y', this.yScale);
        /**
         * We draw the bars into the canvas here. For transitions, the initial height and y have to
         * be set to the bottom of the chart before rendering the actual data. We animate a rising
         * effect, the placement of the transition logic drives what actually animates.
         */
        const svg = this.chart.selectAll('.bar')
            .data(dataSet)
            .enter()
            .append('rect')
            .attr('class', 'bar')
            .attr('width', this.xScale.bandwidth())
            .attr('x', d => this.xScale(d[this.x]))
            .attr('y', this.yScale(0))
            .attr('height', this.chartHeight - this.yScale(0));
        this.tooltipService.addTooltip(svg, this.tooltipLabels, this.chartContainer);
        svg
            .transition()
            .ease(d3.easeLinear)
            .duration(880)
            .attr('y', d => this.yScale(d[this.y]))
            .attr('height', d => this.chartHeight - this.yScale(d[this.y]));
    }
}
BarChartComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngz-charts-barchart',
                template: `
    <div #barChart class="canvas"></div>
  `,
                styles: [`
    .canvas {
      height: 100%;
      width: 100%; }

    ::ng-deep .bar {
      fill: steelblue; }

    ::ng-deep .x-axis, .y-axis {
      font-family: 'Open Sans', sans-serif;
      font-size: 100%;
      font-weight: 400; }
  `]
            },] },
];
/**
 * @nocollapse
 */
BarChartComponent.ctorParameters = () => [
    { type: ChartService, },
    { type: DataService, },
    { type: ToolTipService, },
];
BarChartComponent.propDecorators = {
    'width': [{ type: Input },],
    'height': [{ type: Input },],
    'margins': [{ type: Input },],
    'data': [{ type: Input },],
    'x': [{ type: Input },],
    'y': [{ type: Input },],
    'chartContainer': [{ type: ViewChild, args: ['barChart',] },],
};
function BarChartComponent_tsickle_Closure_declarations() {
    /** @type {?} */
    BarChartComponent.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    BarChartComponent.ctorParameters;
    /** @type {?} */
    BarChartComponent.propDecorators;
    /**
     * Obtain the settings for making the chart here and define defaults if none are specified.
     * @type {?}
     */
    BarChartComponent.prototype.width;
    /** @type {?} */
    BarChartComponent.prototype.height;
    /** @type {?} */
    BarChartComponent.prototype.margins;
    /** @type {?} */
    BarChartComponent.prototype.data;
    /** @type {?} */
    BarChartComponent.prototype.x;
    /** @type {?} */
    BarChartComponent.prototype.y;
    /** @type {?} */
    BarChartComponent.prototype.chartHeight;
    /** @type {?} */
    BarChartComponent.prototype.chartWidth;
    /** @type {?} */
    BarChartComponent.prototype.chart;
    /** @type {?} */
    BarChartComponent.prototype.xScale;
    /** @type {?} */
    BarChartComponent.prototype.yScale;
    /** @type {?} */
    BarChartComponent.prototype.xData;
    /** @type {?} */
    BarChartComponent.prototype.yData;
    /** @type {?} */
    BarChartComponent.prototype.tooltipLabels;
    /** @type {?} */
    BarChartComponent.prototype.chartContainer;
    /** @type {?} */
    BarChartComponent.prototype.chartService;
    /** @type {?} */
    BarChartComponent.prototype.dataService;
    /** @type {?} */
    BarChartComponent.prototype.tooltipService;
}
//# sourceMappingURL=barChart.component.js.map