/**
 * @author Mike Tung <miketung2013@gmail.com>
 * @license
 * Copyright Mike Tung All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file
 */
import { Component, Input, ViewChild } from '@angular/core';
import * as d3 from 'd3';
import { ChartService } from '../shared/chart.service';
import { DataService } from '../shared/data.service';
import { ToolTipService } from '../shared/tooltip.service';
import { Tooltip } from '../shared/models/tooltip';
export class ScatterPlotComponent {
    /**
     * @param {?} chartService
     * @param {?} dataService
     * @param {?} tooltipService
     */
    constructor(chartService, dataService, tooltipService) {
        this.chartService = chartService;
        this.dataService = dataService;
        this.tooltipService = tooltipService;
        /**
         * Setup the chart properties and define defaults if none are specified
         */
        this.width = 600;
        this.height = 600;
        this.margins = { 'top': 50, 'right': 50, 'bottom': 50, 'left': 50 };
        this.dotSize = 5;
        this.dataPointShape = 'circle';
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.data === null) {
            throw new Error('No Data supplied!!');
        }
        this.chartWidth = this.width - this.margins.left - this.margins.right;
        this.chartHeight = this.height - this.margins.top - this.margins.bottom;
        this.tooltipLabels = new Tooltip(this.x, this.y);
        this.chart = this.chartService.makeChartCanvas(this.chartContainer, this.width, this.height, this.margins);
        if (this.data) {
            this.draw(this.data);
        }
    }
    /**
     * This life cycle hook tells angular to detect any changes and to adjust
     * the chart accordingly and also serves to separate out different
     * instances of the same chart
     *
     * @return {?}
     */
    ngOnChanges() {
        this.chartWidth = this.width - this.margins.left - this.margins.right;
        this.chartHeight = this.height - this.margins.top - this.margins.bottom;
        this.tooltipLabels = new Tooltip(this.x, this.y);
        if (this.chart && this.data) {
            this.draw(this.data);
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.width = null;
        this.height = null;
        this.margins = null;
        this.data = null;
        this.x = null;
        this.y = null;
        this.chartHeight = null;
        this.chartWidth = null;
        this.chart = null;
        this.xScale = null;
        this.yScale = null;
        this.xData = null;
        this.yData = null;
    }
    /**
     * Main function to draw the scatter plot, it requires a dataset in order to draw
     * on the reference element supplied from view child
     *
     * \@param{Array<object>} dataSet - data objects in an array.
     * @param {?} dataSet
     * @return {?}
     */
    draw(dataSet) {
        this.xData = this.dataService.getData(dataSet, this.x);
        this.yData = this.dataService.getData(dataSet, this.y);
        this.xScale = this.dataService.makeScale('linear', this.xData, this.chartWidth);
        this.yScale = this.dataService.makeScale('linear', this.yData, this.chartHeight);
        this.makeAxis('x', this.xScale);
        this.makeAxis('y', this.yScale);
        const /** @type {?} */ svg = this.chart.selectAll('.scatter-plot-data')
            .data(dataSet)
            .enter()
            .append(this.dataPointShape)
            .attr('class', 'scatter-plot-data')
            .attr('r', this.dotSize)
            .attr('cx', (d) => this.xScale(d[this.x]))
            .attr('cy', (d) => this.yScale(d[this.y]));
        this.tooltipService.addTooltip(svg, this.tooltipLabels, this.chartContainer);
    }
    /**
     * This makes the axes for the chart.
     *
     *
     * @param {?} axis
     * @param {?} scale
     * @return {?}
     */
    makeAxis(axis, scale) {
        if (axis === 'x') {
            /* This is drawing the x-axis and adding a label*/
            this.chart.append('g')
                .attr('class', 'x-axis')
                .attr('transform', `translate(0, ${this.chartHeight})`)
                .call(d3.axisBottom(scale));
            const /** @type {?} */ movement = `${this.chartWidth / 2}, ${this.chartHeight + this.margins.bottom - 5}`;
            this.chart
                .append('text')
                .attr('transform', `translate(${movement})`)
                .text(`${this.x}`);
        }
        else if (axis === 'y') {
            /* This is drawing the y-axis and adding a label*/
            this.chart.append('g')
                .attr('class', 'y-axis')
                .call(d3.axisLeft(scale));
            this.chart.append('text')
                .attr('transform', 'rotate(-90)')
                .attr('y', `-${this.margins.left}`)
                .attr('x', `-${this.chartHeight / 2}`)
                .attr('dy', '1em')
                .style('text-anchor', 'middle')
                .text(`${this.y}`);
        }
    }
}
ScatterPlotComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngz-charts-scatterplot',
                template: `
    <div #scatterPlot class="scatterplot-canvas"></div>
  `,
                styles: [`
    ::ng-deep .scatterplot-data {
      fill: steelblue; }
  `]
            },] },
];
/**
 * @nocollapse
 */
ScatterPlotComponent.ctorParameters = () => [
    { type: ChartService, },
    { type: DataService, },
    { type: ToolTipService, },
];
ScatterPlotComponent.propDecorators = {
    'width': [{ type: Input },],
    'height': [{ type: Input },],
    'margins': [{ type: Input },],
    'data': [{ type: Input },],
    'x': [{ type: Input },],
    'y': [{ type: Input },],
    'dotSize': [{ type: Input },],
    'dataPointShape': [{ type: Input },],
    'chartContainer': [{ type: ViewChild, args: ['scatterPlot',] },],
};
function ScatterPlotComponent_tsickle_Closure_declarations() {
    /** @type {?} */
    ScatterPlotComponent.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    ScatterPlotComponent.ctorParameters;
    /** @type {?} */
    ScatterPlotComponent.propDecorators;
    /**
     * Setup the chart properties and define defaults if none are specified
     * @type {?}
     */
    ScatterPlotComponent.prototype.width;
    /** @type {?} */
    ScatterPlotComponent.prototype.height;
    /** @type {?} */
    ScatterPlotComponent.prototype.margins;
    /** @type {?} */
    ScatterPlotComponent.prototype.data;
    /** @type {?} */
    ScatterPlotComponent.prototype.x;
    /** @type {?} */
    ScatterPlotComponent.prototype.y;
    /** @type {?} */
    ScatterPlotComponent.prototype.dotSize;
    /** @type {?} */
    ScatterPlotComponent.prototype.dataPointShape;
    /** @type {?} */
    ScatterPlotComponent.prototype.chartHeight;
    /** @type {?} */
    ScatterPlotComponent.prototype.chartWidth;
    /** @type {?} */
    ScatterPlotComponent.prototype.chart;
    /** @type {?} */
    ScatterPlotComponent.prototype.xScale;
    /** @type {?} */
    ScatterPlotComponent.prototype.yScale;
    /** @type {?} */
    ScatterPlotComponent.prototype.xData;
    /** @type {?} */
    ScatterPlotComponent.prototype.yData;
    /** @type {?} */
    ScatterPlotComponent.prototype.tooltipLabels;
    /** @type {?} */
    ScatterPlotComponent.prototype.chartContainer;
    /** @type {?} */
    ScatterPlotComponent.prototype.chartService;
    /** @type {?} */
    ScatterPlotComponent.prototype.dataService;
    /** @type {?} */
    ScatterPlotComponent.prototype.tooltipService;
}
//# sourceMappingURL=scatterPlot.component.js.map