/**
 * @author Mike Tung <miketung2013@gmail.com>
 * @license
 * Copyright Mike Tung All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file
 */
import { Component, Input, ViewChild } from '@angular/core';
import * as d3 from 'd3';
import { Tooltip } from '../shared/models/tooltip';
import { ChartService } from '../shared/chart.service';
import { DataService } from '../shared/data.service';
import { ToolTipService } from '../shared/tooltip.service';
export class TimelineComponent {
    /**
     * @param {?} chartService
     * @param {?} dataService
     * @param {?} tooltipService
     */
    constructor(chartService, dataService, tooltipService) {
        this.chartService = chartService;
        this.dataService = dataService;
        this.tooltipService = tooltipService;
        /**
         * Obtain the settings for making the chart here and define defaults if none are specified.
         */
        this.width = 800;
        this.height = 400;
        this.margins = { 'top': 60, 'right': 60, 'bottom': 60, 'left': 60 };
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        /*Check if data is given if not die*/
        if (this.data == null) {
            throw new Error('Missing Data!');
        }
        this.chartWidth = this.width - this.margins.left - this.margins.right;
        this.chartHeight = this.height - this.margins.top - this.margins.bottom;
        this.tooltipLabels = new Tooltip(this.dateLabel, this.dateEvent);
        this.chart = this.chartService.makeChartCanvas(this.chartContainer, this.width, this.height, this.margins);
        if (this.data) {
            this.draw(this.data);
        }
    }
    /**
     * This life cycle hook tells angular to detect any changes and to adjust
     * the chart accordingly and also serves to separate out different
     * instances of the same chart
     * @return {?}
     */
    ngOnChanges() {
        this.chartWidth = this.width - this.margins.left - this.margins.right;
        this.chartHeight = this.height - this.margins.top - this.margins.bottom;
        this.tooltipLabels = new Tooltip(this.dateLabel, this.dateEvent);
        if (this.chart && this.data) {
            this.draw(this.data);
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.width = null;
        this.height = null;
        this.margins = null;
        this.data = null;
        this.dateLabel = null;
        this.dateEvent = null;
        this.chartHeight = null;
        this.chartWidth = null;
        this.chart = null;
        this.data = null;
        this.timeScale = null;
    }
    /**
     * This makes the timeline axes
     *
     * @param {?} scale
     * @return {?}
     */
    makeTimeAxis(scale) {
        this.chart
            .append('g')
            .attr('class', 'time-axis')
            .attr('transform', `translate(0, ${this.chartHeight / 2})`)
            .call(d3.axisBottom(scale).tickFormat(d3.timeFormat('%m/%d/%Y')));
        const /** @type {?} */ axisLabelPos = `${this.chartWidth / 2}, ${this.chartHeight - this.margins.bottom}`;
        this.chart
            .append('text')
            .attr('transform', `translate(${axisLabelPos})`)
            .text(`${this.dateLabel}`);
    }
    /**
     * This is the main method to render the bar chart it operates on the d3
     * selection supplied and appends the scales, bars to the selection.
     *
     * @param {?} dataSet
     * @return {?}
     */
    draw(dataSet) {
        this.timeData = this.dataService.getData(dataSet, this.dateLabel);
        this.timeScale = this.dataService.makeScale('time', this.timeData, this.chartWidth);
        this.makeTimeAxis(this.timeScale);
        /**
         * We draw the bars into the canvas here. For transitions, the initial height and y have to
         * be set to the bottom of the chart before rendering the actual data. We animate a rising
         * effect, the placement of the transition logic drives what actually animates.
         */
        const svg = this.chart.selectAll('.bar')
            .data(dataSet)
            .enter()
            .append('circle')
            .attr('class', 'time-point')
            .attr('r', 8)
            .attr('cx', d => this.timeScale(d[this.dateLabel]))
            .attr('cy', `${this.chartHeight / 2}`);
        this.tooltipService.addTooltip(svg, this.tooltipLabels, this.chartContainer);
    }
}
TimelineComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngz-charts-timeline',
                template: `
    <div #timeline class="canvas"></div>
  `,
                styles: [`
    .canvas {
      width: 100%;
      height: 100%; }

    ::ng-deep .time-point {
      fill: steelblue; }

    ::ng-deep .time-axis {
      font-family: 'Open Sans', sans-serif;
      font-size: 100%;
      font-weight: 400; }
  `]
            },] },
];
/**
 * @nocollapse
 */
TimelineComponent.ctorParameters = () => [
    { type: ChartService, },
    { type: DataService, },
    { type: ToolTipService, },
];
TimelineComponent.propDecorators = {
    'width': [{ type: Input },],
    'height': [{ type: Input },],
    'margins': [{ type: Input },],
    'data': [{ type: Input },],
    'dateLabel': [{ type: Input },],
    'dateEvent': [{ type: Input },],
    'chartContainer': [{ type: ViewChild, args: ['timeline',] },],
};
function TimelineComponent_tsickle_Closure_declarations() {
    /** @type {?} */
    TimelineComponent.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    TimelineComponent.ctorParameters;
    /** @type {?} */
    TimelineComponent.propDecorators;
    /**
     * Obtain the settings for making the chart here and define defaults if none are specified.
     * @type {?}
     */
    TimelineComponent.prototype.width;
    /** @type {?} */
    TimelineComponent.prototype.height;
    /** @type {?} */
    TimelineComponent.prototype.margins;
    /** @type {?} */
    TimelineComponent.prototype.data;
    /** @type {?} */
    TimelineComponent.prototype.dateLabel;
    /** @type {?} */
    TimelineComponent.prototype.dateEvent;
    /** @type {?} */
    TimelineComponent.prototype.chartHeight;
    /** @type {?} */
    TimelineComponent.prototype.chartWidth;
    /** @type {?} */
    TimelineComponent.prototype.chart;
    /** @type {?} */
    TimelineComponent.prototype.timeScale;
    /** @type {?} */
    TimelineComponent.prototype.timeData;
    /** @type {?} */
    TimelineComponent.prototype.tooltipLabels;
    /** @type {?} */
    TimelineComponent.prototype.chartContainer;
    /** @type {?} */
    TimelineComponent.prototype.chartService;
    /** @type {?} */
    TimelineComponent.prototype.dataService;
    /** @type {?} */
    TimelineComponent.prototype.tooltipService;
}
//# sourceMappingURL=timeline.component.js.map